package java.net;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.util.Iterator;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.security.Permission;

public final class URLPermission extends Permission
{
    private static final long serialVersionUID = -2702463814894478682L;
    private transient String scheme;
    private transient String ssp;
    private transient String path;
    private transient List<String> methods;
    private transient List<String> requestHeaders;
    private transient URLPermission.Authority authority;
    private String actions;
    
    public URLPermission(String name, String s) {
        super(name);
        this.init(s);
    }
    
    private void init(String substring) {
        this.parseURI(this.getName());
        int index = substring.indexOf(58);
        if (substring.lastIndexOf(58) != index) {
            throw new IllegalArgumentException("Invalid actions string: \"" + substring + "\"");
        }
        String substring2;
        if (index == -1) {
            substring2 = "";
        }
        else {
            substring = substring.substring(0, index);
            substring2 = substring.substring(index + 1);
        }
        List normalizeMethods = this.normalizeMethods(substring);
        Collections.sort(normalizeMethods);
        this.methods = Collections.unmodifiableList(normalizeMethods);
        List normalizeHeaders = this.normalizeHeaders(substring2);
        Collections.sort(normalizeHeaders);
        this.requestHeaders = Collections.unmodifiableList(normalizeHeaders);
        this.actions = this.actions();
        return;
    }
    
    public URLPermission(String s) {
        this(s, "*:*");
    }
    
    @Override
    public String getActions() {
        return this.actions;
    }
    
    @Override
    public boolean implies(Permission permission) {
        if (!(permission instanceof URLPermission)) {
            return false;
        }
        URLPermission urlPermission = (URLPermission)permission;
        if (!((String)this.methods.get(0)).equals("*") && Collections.indexOfSubList(this.methods, urlPermission.methods) == -1) {
            return false;
        }
        if (this.requestHeaders.isEmpty() && !urlPermission.requestHeaders.isEmpty()) {
            return false;
        }
        if (!this.requestHeaders.isEmpty() && !((String)this.requestHeaders.get(0)).equals("*") && Collections.indexOfSubList(this.requestHeaders, urlPermission.requestHeaders) == -1) {
            return false;
        }
        if (!this.scheme.equals(urlPermission.scheme)) {
            return false;
        }
        if (this.ssp.equals("*")) {
            return true;
        }
        if (!this.authority.implies(urlPermission.authority)) {
            return false;
        }
        if (this.path == null) {
            return urlPermission.path == null;
        }
        if (urlPermission.path == null) {
            return false;
        }
        if (this.path.endsWith("/-")) {
            return urlPermission.path.startsWith(this.path.substring(0, this.path.length() - 1));
        }
        if (this.path.endsWith("/*")) {
            String substring = this.path.substring(0, this.path.length() - 1);
            if (!urlPermission.path.startsWith(substring)) {
                return false;
            }
            String substring2 = urlPermission.path.substring(substring.length());
            return substring2.indexOf(47) == -1 && !substring2.equals("-");
        }
        else {
            return this.path.equals(urlPermission.path);
        }
    }
    
    @Override
    public boolean equals(Object o) {
        if (!(o instanceof URLPermission)) {
            return false;
        }
        URLPermission urlPermission = (URLPermission)o;
        if (!this.scheme.equals(urlPermission.scheme)) {
            return false;
        }
        if (!this.getActions().equals(urlPermission.getActions())) {
            return false;
        }
        if (!this.authority.equals(urlPermission.authority)) {
            return false;
        }
        if (this.path != null) {
            return this.path.equals(urlPermission.path);
        }
        return urlPermission.path == null;
    }
    
    @Override
    public int hashCode() {
        return this.getActions().hashCode() + this.scheme.hashCode() + this.authority.hashCode() + ((this.path == null) ? 0 : this.path.hashCode());
    }
    
    private List<String> normalizeMethods(String str) {
        ArrayList<String> list = new ArrayList();
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < str.length(); ++i) {
            int char1 = str.charAt(i);
            if (char1 == ',') {
                String string = sb.toString();
                if (string.length() > 0) {
                    list.add(string);
                }
                sb = new StringBuilder();
            }
            else {
                if (char1 == ' ' || char1 == '\t') {
                    throw new IllegalArgumentException("White space not allowed in methods: \"" + str + "\"");
                }
                if (char1 >= 'a' && char1 <= 'z') {
                    char1 = (char)(char1 - 32);
                }
                sb.append((char)char1);
            }
        }
        String string2 = sb.toString();
        if (string2.length() > 0) {
            list.add(string2);
            return list;
        }
        return list;
    }
    
    private List<String> normalizeHeaders(String str) {
        ArrayList<String> list = new ArrayList();
        StringBuilder sb = new StringBuilder();
        int n = 1;
        for (int i = 0; i < str.length(); ++i) {
            int char1 = str.charAt(i);
            if (char1 >= 'a' && char1 <= 'z') {
                if (n != 0) {
                    char1 = (char)(char1 - 32);
                    n = 0;
                }
                sb.append((char)char1);
            }
            else {
                if (char1 == ' ' || char1 == '\t') {
                    throw new IllegalArgumentException("White space not allowed in headers: \"" + str + "\"");
                }
                if (char1 == '-') {
                    n = 1;
                    sb.append((char)char1);
                }
                else if (char1 == ',') {
                    String string = sb.toString();
                    if (string.length() > 0) {
                        list.add(string);
                    }
                    sb = new StringBuilder();
                    n = 1;
                }
                else {
                    n = 0;
                    sb.append((char)char1);
                }
            }
        }
        String string2 = sb.toString();
        if (string2.length() > 0) {
            list.add(string2);
            return list;
        }
        return list;
    }
    
    private void parseURI(String s) {
        int length = s.length();
        int index = s.indexOf(58);
        if (index == -1 || index + 1 == length) {
            throw new IllegalArgumentException("Invalid URL string: \"" + s + "\"");
        }
        this.scheme = s.substring(0, index).toLowerCase();
        this.ssp = s.substring(index + 1);
        if (!this.ssp.startsWith("//")) {
            if (!this.ssp.equals("*")) {
                throw new IllegalArgumentException("Invalid URL string: \"" + s + "\"");
            }
            this.authority = new URLPermission.Authority(this.scheme, "*");
            return;
        }
        else {
            String s2 = this.ssp.substring(2);
            int index2 = s2.indexOf(47);
            if (index2 == -1) {
                this.path = "";
            }
            else {
                s2 = s2.substring(0, index2);
                this.path = s2.substring(index2);
            }
            this.authority = new URLPermission.Authority(this.scheme, s2.toLowerCase());
            return;
        }
    }
    
    private String actions() {
        StringBuilder sb = new StringBuilder();
        Iterator<String> iterator = this.methods.iterator();
        while (iterator.hasNext()) {
            sb.append((String)iterator.next());
        }
        sb.append(":");
        Iterator<String> iterator2 = this.requestHeaders.iterator();
        while (iterator2.hasNext()) {
            sb.append((String)iterator2.next());
        }
        return sb.toString();
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        this.init((String)objectInputStream.readFields().get("actions", null));
    }
    
    static class Authority
    {
        HostPortrange p;
        
        Authority(String s, String s2) {
            int index = s2.indexOf(64);
            if (index == -1) {
                this.p = new HostPortrange(s, s2);
            }
            else {
                this.p = new HostPortrange(s, s2.substring(index + 1));
            }
        }
        
        boolean implies(URLPermission.Authority authority) {
            return this.impliesHostrange(authority) && this.impliesPortrange(authority);
        }
        
        private boolean impliesHostrange(URLPermission.Authority authority) {
            String hostname = this.p.hostname();
            String hostname2 = authority.p.hostname();
            return (this.p.wildcard() && hostname.equals("")) || ((!authority.p.wildcard() || !hostname2.equals("")) && (hostname.equals(hostname2) || (this.p.wildcard() && hostname2.endsWith(hostname))));
        }
        
        private boolean impliesPortrange(URLPermission.Authority authority) {
            int[] portrange = this.p.portrange();
            int[] portrange2 = authority.p.portrange();
            return portrange[0] == -1 || (portrange[0] <= portrange2[0] && portrange[1] >= portrange2[1]);
        }
        
        boolean equals(URLPermission.Authority authority) {
            return this.p.equals(authority.p);
        }
        
        @Override
        public int hashCode() {
            return this.p.hashCode();
        }
    }
}
